#include <View.h>
#include <Application.h>
#include <Window.h>
#include <Menubar.h>
#include <Menu.h>
#include <MenuItem.h>
#include "Tetris.h"
#include "TetrisView.h"
#include "BTetris.h"

main()
{
	TetrisApplication *app;
	app=new TetrisApplication;
	app->Run();
	delete(app);
	return(0);
}

TetrisApplication::TetrisApplication() : BApplication('TTRS')
{
	BRect wrect;
	abouting=0;
	paused=0;
	
	wrect.Set(50,50,500,500);  // Arbitary except for setting top left corner
    win=new TetrisWindow(wrect);
	win->Show();

} 

void TetrisApplication::AboutRequested(void)
{
	if(!abouting) {
		abouting=1;
		paused=1;
		win->Pause();		
		AboutWindow *aw;
 		BRect rect(100,100,350,450);
		aw=new AboutWindow(rect);
		aw->Show();
	}
}

void TetrisApplication::MessageReceived(BMessage *msg)
{
	be_app->Lock();
	switch(msg->what) {
		case BT_PAUSE:
			paused=1;
			win->Pause();
			break;
		case BT_UNPAUSE:
			win->Continue();
			paused=0;
			break;
		case BT_TPAUSE:
			if(paused) {	
				win->Continue();
				paused=0;
			} else {
				win->Pause();
				paused=1;
			}
			break;
		case BT_NEW:
 			win->Restart();
			break;	 
		case BT_END:
			win->End();	 
			break;
		case BT_SPEEDUP:
			win->Speedup();
	}
	be_app->Unlock();
}

//----------------------------------------------------------------------

AboutWindow::AboutWindow(BRect frame) : BWindow(frame,"About Betris",B_TITLED_WINDOW,B_NOT_RESIZABLE)
{	
	rgb_color red={128,0,0,0};
	rgb_color blue={0,0,128,0};
	BFont font;
	const char *titletext="BeTris\nVersion " VERSION "\nwritten by Matthew Crosby\ncrosby@cs.colorado.edu";
	BTextView *title;
	BTextView *usage;
	BTextView *copyright;
	BButton *okb;
	// TITLE:
	BRect rect(0,0,250,120);
	BRect ir(0,0,250,120);
    title=new BTextView(rect,"Title",ir,B_FOLLOW_ALL,B_WILL_DRAW);
    title->MakeEditable(false);
    title->SetText(titletext,strlen(titletext));
    title->SetAlignment(B_ALIGN_CENTER);
    title->SetStylable(true);
    font.SetSize(50);
    font.SetFamilyAndStyle("Baskerville","Italic");
    title->SetFontAndColor(0,1,&font,B_FONT_ALL,&blue);
    title->SetFontAndColor(1,2,&font,B_FONT_ALL,&red);
    title->SetFontAndColor(2,6,&font);
    font.SetSize(14);
    title->SetFontAndColor(6,strlen(titletext),&font,B_FONT_SIZE);
    title->MakeSelectable(false);
    AddChild(title);
    // USAGE:
    rect.Set(0,130,250,250);
	ir.Set(40,0,200,160);
    usage=new BTextView(rect,"Usage",ir,B_FOLLOW_ALL,B_WILL_DRAW);
    usage->SetText(USAGE,sizeof(USAGE));
    usage->MakeEditable(false);
    usage->MakeSelectable(false);
    AddChild(usage);
    //title->Bounds();
    // COPRIGHT:
    rect.Set(0,265,250,325);
    ir.Set(10,0,250,50);
    copyright=new BTextView(rect,"Copyright",ir,B_FOLLOW_ALL,B_WILL_DRAW);
    copyright->MakeEditable(false);
    copyright->MakeSelectable(false);
    copyright->SetText(COPYRIGHT,strlen(COPYRIGHT));
    font.SetSize(9);
    copyright->SetFontAndColor(&font,B_FONT_SIZE);
    AddChild(copyright);
    // OK Button:
    rect.Set(100,325,150,350);
    okb=new BButton(rect,"OK","OK",new BMessage(B_QUIT_REQUESTED));
	AddChild(okb);
}

bool AboutWindow::QuitRequested()
{
	((TetrisApplication *)be_app)->abouting=0;
	be_app->PostMessage(BT_UNPAUSE);
	return(TRUE);
}

//------------------------------------------------------------------------

TetrisWindow::TetrisWindow(BRect frame) : BWindow(frame,"Betris",B_TITLED_WINDOW, B_NOT_RESIZABLE)
{
	BRect wrect;
	PulseRate=START_PULSE_RATE;
	SetPulseRate(START_PULSE_RATE);
	    wrect.Set(0,0,500,500);
    sview=new TetrisWindowView(wrect,"StatusView");
	AddChild(sview);
	wrect=sview->Bounds();
	
	ResizeTo(wrect.Width(),wrect.Height());
}

void TetrisWindow::Pause(void)
{
	Lock();
	sview->Pause();
	SetPulseRate(0);
	Unlock();
}

void TetrisWindow::Continue(void)
{
	Lock();
	sview->Continue();
	SetPulseRate(PulseRate);
	Unlock();
}

void TetrisWindow::Restart(void)
{
	Lock();
 	SetPulseRate(0);
	sview->Restart();
	PulseRate=START_PULSE_RATE;
	SetPulseRate(START_PULSE_RATE);
 	Unlock();
}

void TetrisWindow::End(void)
{
	Lock();	
    SetPulseRate(0);
	sview->End();
	Unlock();
}
void TetrisWindow::Speedup(void)
{
	Lock();
	PulseRate-=PULSE_INCREMENT;
	if(PulseRate<=0) {
		PulseRate=1;
	}
	SetPulseRate(PulseRate);
	Unlock();
}
 
bool TetrisWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(TRUE);
}

//-----------------------------------------------------------------------

TetrisWindowView::TetrisWindowView(BRect rect, char *name) : 
    BView(rect, name, B_FOLLOW_LEFT,B_WILL_DRAW | B_PULSE_NEEDED)
{
	
	BRect rct;	
	BMenu *gamemenu;
	BMenuItem *item;	
	
	// Create menubar
    rct.Set(0,0,65536,20);  // Very arbitary rectangle
    menubar=new BMenuBar(rct,"MenuBar"); // Add the menu bar
    SetViewColor(menubar->ViewColor());
  
    // Populate menu bar
    gamemenu=new BMenu("Game");
	menubar->AddItem(gamemenu);
    AddChild(menubar);
    item=new BMenuItem("About",new BMessage(B_ABOUT_REQUESTED));
    item->SetTarget(be_app);
    gamemenu->AddItem(item);
    item=new BMenuItem("Pause",new BMessage(BT_TPAUSE));
    item->SetTarget(be_app);
    gamemenu->AddItem(item);
    PauseItem=item;
   // GetParent()->PauseItem=item;
    item=new BMenuItem("New Game",new BMessage(BT_NEW));
    item->SetTarget(be_app);
    gamemenu->AddItem(item);
    item=new BMenuItem("Quit",new BMessage(B_QUIT_REQUESTED));
    gamemenu->AddItem(item);
    
    // Create the Status view
    
	rct.Set(0,0,100,40);
	statusView=new TetrisStatusView(rct,"Status View");
	AddChild(statusView);
    // Add Tetris frame itself.
    rct.Set(0,0,100,100); // Fairly arbitary, in fact it resizes itselfx
	view=new TetrisView(rct,"Main Tetris");	
	AddChild(view);
}
 
void TetrisWindowView::AllAttached(void)
{
	BRect rect;
	float height;
	float width;
	
	rect=menubar->Bounds();
	height=rect.Height();
	statusView->MoveTo(0,height+1);
	height+=40;
	view->MoveTo(0,height+1);	
	rect=view->Bounds();	
	height+=rect.Height();
	width=rect.Width();
	statusView->ResizeTo(width,30);
	ResizeTo(width,height);
}

void TetrisWindowView::Pause(void)
{
	PauseItem->SetMarked(true);
	view->Pause();
}

void TetrisWindowView::Continue(void)
{
	view->Continue();
	PauseItem->SetMarked(false);
}

void TetrisWindowView::Restart(void)
{
 	view->Reset();
	statusView->Invalidate();
}

void TetrisWindowView::End(void)
{
	view->GameOver();
	statusView->Invalidate();
}

void TetrisWindowView::DrawNext(void)
{
	statusView->DrawNext();
}

void TetrisWindowView::DrawScore(void)
{
	statusView->DrawScore();
}
