#include <View.h>

#include "Tetris.h"
#include "TetrisView.h"
#include "BTetris.h"

Tetris piece; 

int pulsing=0;
rgb_color Black={0,0,0,0};
rgb_color ClearC={255,0,0,0};
struct {
	rgb_color main;
	rgb_color top_shadow;
	rgb_color bottom_shadow;
} Color_Table[SHAPE_COUNT+1] = {
{{255,255,255,0},{255,255,255,0},{255,255,255,0}},
{{255,62,150,0},{238,58,140,0},{205,41,144}},
{{255,127,0,0},{238,118,0,0},{205,102,0,0}},
{{255,0,0,0},{238,0,0,0},{205,0,0,0}},
{{64,20,147,0},{54,18,137,0},{44,16,118,0}},  
{{0,128,0,0},{0,111,0,0},{0,94,0,0}},
{{0,0,255,0},{0,0,238,0},{0,0,205,0}},
{{255,193,37,0},{238,201,0,0},{205,155,29,0}}};


//-----------------------------------------------------------------------

TetrisView::TetrisView(BRect rect, char *name) 
           : BView(rect,name,B_FOLLOW_LEFT,B_WILL_DRAW | B_PULSE_NEEDED)
{
	 ResizeTo(WIDTH*EX,HEIGHT*EY);
}

void TetrisView::Pulse()
{	
	Rect exposed;
	BRect Bexposed;
	if(!playing) {
		return;
	}
	if(!piece.Advance(&exposed)) {
		DrawCleared(piece.GetCleared());
		exposed.Set(0,0,WIDTH,HEIGHT);
	}
	Bexposed.RtoBR(exposed);
	Draw(Bexposed);
	if(piece.WellFull()) {
		be_app->PostMessage(BT_END);
	}
	be_app->PostMessage(BT_SPEEDUP); // later we'll be more sophisticated.
}

void TetrisView::KeyDown(const char *bytes, int32 numBytes)
{
	Rect exposed(0,0,0,0);
	BRect Bexposed;
	
	if( numBytes==1 ) {
		switch(bytes[0]) {
			case 'q':
				be_app->PostMessage(B_QUIT_REQUESTED);
				break;
		case 'n':
			be_app->PostMessage(BT_NEW);
		    return;
		}
		
		if(paused) {
			be_app->PostMessage(BT_UNPAUSE);
			return;
		} else if(!playing) {
			return;
		}
	
		switch (bytes[0]) {
			case 'p':
	 			be_app->PostMessage(BT_TPAUSE);
				break;
			case B_RIGHT_ARROW:
				if(!paused) { 
					piece.Move(Right,&exposed);
				}
				break;
			case B_LEFT_ARROW:
				if(!paused) {
					piece.Move(Left,&exposed);
				}
				break;
			case B_UP_ARROW:
				if(!paused) {
					piece.Rotate(Left,&exposed);
				}
				break;
			case B_DOWN_ARROW:
				if(!paused) {
					piece.Rotate(Right,&exposed);
				}
				break;
			case B_SPACE:
				if(!paused)
				{
					while(piece.Advance(&exposed)) {
						Bexposed.RtoBR(exposed);
						Draw(Bexposed);
					}
					DrawCleared(piece.GetCleared());
					exposed.Set(0,0,WIDTH,HEIGHT);
				}
				break;
		}
		if(exposed.xmax!=0) {
			Bexposed.RtoBR(exposed);
			Draw(Bexposed);
			Flush();
		}
	}
}



void TetrisView::AttachedToWindow()
{
	MakeFocus(true);
	playing=true;
}

void TetrisView::Draw(BRect exposed)
{
	BRect rect;
	BPoint p1,p2,p3,p4;
	int x,y;
	rect.Set(1,1,EX-1,EY-1);
	for(x=0;x<WIDTH;x++) 
		for(y=0;y<HEIGHT;y++)
		{
			rect.OffsetTo(x*EX,y*EY);
			if(exposed.Contains(rect)) {
				SetHighColor(Color_Table[piece.OccupiedBy(x,y)].main);
				FillRect(rect);
					p1.Set(x*EX,y*EY);
					p2.Set(x*EX,y*EY+EY-1);
					p3.Set(x*EX+EX-1,y*EY);
					p4.Set(x*EX+EX-1,y*EY+EY-1);
					SetHighColor(Color_Table[piece.OccupiedBy(x,y)].top_shadow);
					StrokeLine(p1,p2);
					StrokeLine(p1,p3);
					SetHighColor(Color_Table[piece.OccupiedBy(x,y)].bottom_shadow);
					StrokeLine(p4,p2);
					StrokeLine(p4,p3);
			}
		}
	if(paused) {
		DrawPaused();
	}
	if(!playing) {
		DrawOver();
	}
}

void TetrisView::DrawPaused(void)
{
	BPoint point;
	BRect frame;
	frame=Bounds();
	SetFontSize(40);
	SetHighColor(Black);
	point.Set((frame.Width()/2)-(StringWidth("Paused")/2),100);
	DrawString("Paused"	,point);
}

void TetrisView::DrawOver(void)
{
	BPoint point;
	BRect frame;
	frame=Bounds();
	SetFontSize(35);
	SetHighColor(Black);
	point.Set((frame.Width()/2)-(StringWidth("Game Over")/2),100);
	DrawString("Game Over",point);
	SetFontSize(15);
	point.Set((frame.Width()/2)-(StringWidth("Press n for a new game")/2),150);
	DrawString("Press n for a new game",point);
}

void TetrisView::DrawCleared(bool *cleared)
{
	bool dome=false;
	BRect rect;
	int y;
	((TetrisWindowView *)Parent())->DrawNext();
	((TetrisWindowView *)Parent())->DrawScore();
	rect.Set(0,0,WIDTH*EX,EY);
	for(y=0;y<HEIGHT;y++) {
		if(cleared[y]) {
			rect.OffsetTo(0,y*EY);
			SetHighColor(ClearC);
			FillRect(rect);
			dome=true;
		}
	}
	if(dome) {
		Sync();
		snooze(400000);
		for(y=0;y<HEIGHT;y++) {
			if(cleared[y]) {
				rect.OffsetTo(0,y*EY);
				SetHighColor(LowColor());
				FillRect(rect);
			}
		}
	}
}

void TetrisView::Reset(void)
{
 	BRect rect=Bounds();
	piece.Restart();
	Invalidate();
	playing=true;
}

void TetrisView::Pause(void)
{
	paused=true;
	DrawPaused();
}

void TetrisView::Continue(void)
{
	BRect nr;
	nr=Frame();
	paused=false;
	Draw(nr);

}

void TetrisView::GameOver(void)
{
 	playing=false;
	DrawOver();
}

//---------------------------------------------------------------


TetrisStatusView::TetrisStatusView(BRect rect, char *name) : 
    BView(rect, name, B_FOLLOW_LEFT,B_WILL_DRAW | B_PULSE_NEEDED)
{
}


void TetrisStatusView::AttachedToWindow(void)
{
    SetViewColor(Parent()->ViewColor());
}

void TetrisStatusView::Draw(BRect exposed)
{
	BPoint point;
	font_height font_height;
	SetFontSize(14);
	GetFontHeight(&font_height);
	yfontpos=(NEXTY1+NEXTSIZE/2)+(font_height.ascent/2);
	point.Set(4,yfontpos);
	SetHighColor(Black);
	DrawString("Next:",point);
	NEXTX1=StringWidth("Next:")+4+4;	
	NEXTX2=NEXTX1+NEXTSIZE;
	DrawNext();
	SetHighColor(Black);
	point.Set(100,yfontpos);
	DrawString("Score:",point);
	xscorepos=100+StringWidth("Score:")+5;
	DrawScore();
}	
	
void TetrisStatusView::DrawNext()
{
	BRect nrect;
	int i,j,x,y,xsize,ysize;
	char buffer[SWIDTH][SHEIGHT];
	piece.GetShape(piece.GetNext(),buffer,&x,&y);
 
	xsize=NEXTSIZE/x;
	ysize=NEXTSIZE/y;
     nrect.Set(NEXTX1,NEXTY1,NEXTX2,NEXTY2);
     SetHighColor(ViewColor());
    FillRect(nrect);
    
	for(i=0;i<x;i++)
	{
		for(j=0;j<y;j++)
		{
			nrect.Set((i*xsize)+NEXTX1,(j*ysize)+NEXTY1,((i+1)*xsize)-1+NEXTX1,((j+1)*ysize)-1+NEXTY1);
			if(buffer[j][i])
			    SetHighColor(Color_Table[buffer[j][i]].main);
			else
			 	SetHighColor(ViewColor());
			FillRect(nrect);
		}
	}
}



void TetrisStatusView::DrawScore()
{
	int score;
    BPoint point;
	BRect rect;
	char string[255];
	SetFontSize(14);
	rect.Set(xscorepos-3,NEXTY1-2,xscorepos+scorewidth+3,NEXTY2+2);
	SetHighColor(ViewColor());
	FillRect(rect);
	score=piece.GetScore();
	sprintf(string,"%d",score);
	point.Set(xscorepos,yfontpos);
	SetHighColor(Black);
	DrawString(string,point);	
	scorewidth=StringWidth(string);
}



